package com.uas.app.city.account.config.interceptors;

import com.uas.app.city.account.model.LocalUser;
import com.uas.app.city.account.model.UserView;
import com.uas.app.city.account.service.LocalUserService;
import com.uas.app.city.account.support.SystemSession;
import com.uas.app.city.account.util.FlexJsonUtils;
import com.uas.app.city.account.util.sso.AccessUtils;
import com.uas.sso.SSOConfig;
import com.uas.sso.SSOHelper;
import com.uas.sso.SSOToken;
import com.uas.sso.entity.UserAccount;
import com.uas.sso.web.spring.AbstractSSOInterceptor;
import org.apache.commons.lang3.Validate;
import org.apache.log4j.Logger;
import org.springframework.http.HttpStatus;
import org.springframework.ui.ModelMap;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * SSO单点登录身份信息鉴权拦截器.
 *
 * @author huxz
 */
public class SsoInterceptor extends AbstractSSOInterceptor {

    /**
     * 认证信息
     */
    private static final String AUTHENTICATION_URL = "/user/authentication";

    private final Logger logger = Logger.getLogger(SsoInterceptor.class);

    private final LocalUserService userService;

    public SsoInterceptor(LocalUserService userService) {
        this.userService = userService;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response,
            Object handler, Exception ex) throws Exception {
        try {
            super.afterCompletion(request, response, handler, ex);
        } finally {
            SystemSession.clear();
        }
    }

    @Override
    protected boolean onAuthenticateFailed(HttpServletRequest request,
            HttpServletResponse response) {
        if (logger.isDebugEnabled()) {
            logger.debug("Request uri: " + request.getRequestURI());
        }

        SystemSession.clear();

        if (request.getRequestURI().endsWith(AUTHENTICATION_URL)) {
            return true;
        }
        response.setStatus(HttpStatus.UNAUTHORIZED.value());
        if (!isRedirectAble(request)) {
            try {
                printJson(response, new ModelMap("loginUrl", getLoginPage(request, response)));
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return false;
    }

    @Override
    protected void onAuthenticateSuccess(HttpServletRequest request, HttpServletResponse response) {
        SSOToken token = SSOHelper.attrToken(request);
        assert token != null;

        // 用于鉴别个人用户或企业用户是否入驻当前发起请求的产城门户网站所隶属产城
        final String defaultCity = "5";
        String cityId = request.getParameter("cityId");
        if (StringUtils.isEmpty(cityId)) {
            // TODO 此处需要修改
            cityId = defaultCity;
        }

        LocalUser user = (LocalUser) request.getSession().getAttribute("user");
        if (AccessUtils.needUpdateUserSession(token, user)) {
            user = userService.findUserInfoByToken(token, cityId);

            if (user != null) {
                // 更新当前用户会话中用户信息
                request.getSession().setAttribute("user", user);
            }
        } else {
            logger.info("Don't need update user info");
        }

        Validate.notNull(user, "用户没有注册或用户信息出现异常");

        UserAccount userAccount = FlexJsonUtils.fromJson(token.getData(), UserAccount.class);
        UserView tokenUser = new UserView(userAccount);
        user.setUser(tokenUser);
        SystemSession.setUser(user);
    }

    /**
     * 输出json格式
     */
    private void printJson(HttpServletResponse response, Object obj) throws IOException {
        response.addHeader("Content-Type", "application/json; charset=UTF-8");
        PrintWriter printWriter = response.getWriter();
        printWriter.append(FlexJsonUtils.toJson(obj));
        printWriter.flush();
        printWriter.close();
    }

    private String getLoginPage(HttpServletRequest request, HttpServletResponse response)
            throws IOException {
        request.getSession().setAttribute(SSOConfig.SSOReferer, request.getHeader("Referer"));
        SSOHelper.clearLogin(request, response);
        String redirectUrl = SSOHelper.getRedirectRefererLoginUrl(request);
        boolean cross = SSOHelper.isCrossDomain(request);
        if (cross) {
            // 跨域代理界面
            redirectUrl = request.getContextPath() + "/login/proxy";
        }
        return redirectUrl;
    }
}
