package com.uas.app.city.account.config.zuul;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import org.apache.log4j.Logger;
import org.springframework.cloud.netflix.zuul.filters.RefreshableRouteLocator;
import org.springframework.cloud.netflix.zuul.filters.SimpleRouteLocator;
import org.springframework.cloud.netflix.zuul.filters.ZuulProperties;
import org.springframework.cloud.netflix.zuul.filters.ZuulProperties.ZuulRoute;
import org.springframework.jdbc.core.ColumnMapRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

/**
 * Custom route locator based on {@link SimpleRouteLocator }.
 *
 * @author huxz
 */
public class CustomRouteLocator extends SimpleRouteLocator implements RefreshableRouteLocator {

    private static final Logger logger = Logger.getLogger(CustomRouteLocator.class);

    private ZuulProperties properties;

    private JdbcTemplate jdbcTemplate;

    public CustomRouteLocator(String servletPath,
            ZuulProperties properties) {
        super(servletPath, properties);
        this.properties = properties;
    }

    public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    @Override
    public void refresh() {
        doRefresh();
    }

    @Override
    protected Map<String, ZuulRoute> locateRoutes() {
        LinkedHashMap<String, ZuulRoute> routesMap = new LinkedHashMap<>();
        // 获取 application.yml 属性文件中的路由配置信息
        routesMap.putAll(super.locateRoutes());
        // 获取 数据库 中路由配置信息
        routesMap.putAll(locateRoutesFromDB());

        // 添加全局敏感头部过滤
        Set<String> ignoredHeaders = this.properties.getIgnoredHeaders();
        for (Entry<String, ZuulRoute> routeEntry : routesMap.entrySet()) {
            routeEntry.getValue().getSensitiveHeaders().addAll(ignoredHeaders);
        }

        return routesMap;
    }

    private Map<String, ZuulRoute> locateRoutesFromDB() {
        if (logger.isDebugEnabled()) {
            logger.debug("Load dynamic routes data from database");
        }

        LinkedHashMap<String, ZuulRoute> routes = new LinkedHashMap<>();
        List<Map<String, Object>> results = jdbcTemplate
                .query("select city_domain, admin_site from city", new ColumnMapRowMapper());

        if (!CollectionUtils.isEmpty(results)) {
            for (Map<String, Object> result : results) {
                String id = (String) result.getOrDefault("city_domain", "");
                String url = (String) result.getOrDefault("admin_site", "");

                if (StringUtils.isEmpty(id) || StringUtils.isEmpty(url)) {
                    continue;
                }

                ZuulRoute route = new ZuulRoute();
                route.setId(id);
                route.setPath("/city/" + id + "/**" );
                route.setUrl(url);
                route.setStripPrefix(true);

                routes.put(route.getPath(), route);
            }
        }
        return routes;
    }

}
