package com.uas.app.city.account.config.zuul.filters;

import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import com.uas.app.city.account.model.LocalUser;
import com.uas.app.city.account.model.UserView;
import com.uas.app.city.account.support.SystemSession;
import com.uas.app.city.account.third.model.AccessToken;
import com.uas.app.city.account.third.service.AccessTokenService;
import com.uas.app.city.account.util.FlexJsonUtils;
import com.uas.sso.SSOHelper;
import com.uas.sso.SSOToken;
import com.uas.sso.entity.UserAccount;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.log4j.Logger;
import org.springframework.http.HttpStatus;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.*;
import java.util.Map.Entry;

import static org.springframework.cloud.netflix.zuul.filters.support.FilterConstants.PRE_TYPE;

/**
 * A implementation of {@link ZuulFilter} for authentication.
 *
 * @author huxz
 */
public class AccessPreFilter extends ZuulFilter {

    public static final Logger logger = Logger.getLogger(AccessPreFilter.class);

    private final AccessTokenService accessTokenService;

    public AccessPreFilter(
            AccessTokenService accessTokenService) {
        this.accessTokenService = accessTokenService;
    }

    @Override
    public String filterType() {
        return PRE_TYPE;
    }

    @Override
    public int filterOrder() {
        return 0;
    }

    @Override
    public boolean shouldFilter() {
        RequestContext context = RequestContext.getCurrentContext();
        HttpServletRequest request = context.getRequest();

        logger.info(String.format("proxy request: %s?%s", request.getRequestURI(),
                request.getQueryString()));

        return !request.getRequestURI().contains("/api/serve/mainPage")
                && !request.getRequestURI().contains("/api/common/file/upload");
    }

    @Override
    public Object run() {
        RequestContext context = RequestContext.getCurrentContext();
        HttpServletRequest request = context.getRequest();

        logger.info("==== Start to verify user info ====");

        // 验证用户是否登录
        SSOToken token = SSOHelper.getToken(request);
        if (token == null) {
            context.setSendZuulResponse(false);
            context.setResponseStatusCode(HttpStatus.UNAUTHORIZED.value());
            return null;
        }

        if (logger.isDebugEnabled()) {
            logger.debug("Token data: " + token.getData());
        }

        // 验证是否提取用户信息
        UserAccount userAccount = FlexJsonUtils.fromJson(token.getData(), UserAccount.class);
        UserView tokenUser = new UserView(userAccount);
        if (tokenUser == null) {
            context.setSendZuulResponse(false);
            context.setResponseStatusCode(HttpStatus.UNAUTHORIZED.value());
            return null;
        }

        // 从Token中获取用户基本信息
        LocalUser user = new LocalUser();
        user.setUserUu(NumberUtils.toLong(tokenUser.getDialectUID()));
        user.setUser(tokenUser);
        SystemSession.setUser(user);
        if (logger.isDebugEnabled()) {
            logger.debug("User info: " + FlexJsonUtils.toJson(user));
        }

        // 生成UAS系统的Access Token
        AccessToken accessToken = accessTokenService.createNew("10046669");
        if (logger.isDebugEnabled()) {
            logger.debug("Access token: " + accessToken.getId());
        }

        // 填充Access Token到请求参数中
        Map<String, List<String>> paramsMap = extractParamsFromRequest(request);
        paramsMap.put("client_type", Collections.singletonList("cc"));
        paramsMap.put("access_token", Collections.singletonList(accessToken.getId()));

        System.out.println("==================== Begin ========================");
        for (Entry<String, List<String>> entry : paramsMap.entrySet()) {
            if (entry.getValue() != null && entry.getValue().size() > 0) {
                System.out.println(entry.getKey() + ": " + FlexJsonUtils.toJson(entry.getValue()));
            }
        }
        System.out.println("==================== End ========================");

        context.setRequestQueryParams(paramsMap);

        logger.info("==== End verification ====");
        return null;
    }

    private Map<String, List<String>> extractParamsFromRequest(HttpServletRequest request) {
        Map<String, String[]> map = request.getParameterMap();

        Map<String, List<String>> paramsMap = new HashMap<>(16);
        for (Entry<String, String[]> entry : map.entrySet()) {
            if (entry.getValue() != null && entry.getValue().length > 0) {
                List<String> list = Arrays.asList(entry.getValue());
                if ("data".equals(entry.getKey())) {
                    ArrayList<String> values = new ArrayList<>();
                    for (String value : list) {
                        try {
                            values.add(URLDecoder.decode(value, "UTF-8"));
                        } catch (UnsupportedEncodingException e) {
                            e.printStackTrace();
                        }
                    }
                    list = values;
                }
                paramsMap.put(entry.getKey(), list);
            }
        }
        return paramsMap;
    }

}
