package com.uas.app.city.account.service.impl;

import com.uas.app.city.account.dao.CityDao;
import com.uas.app.city.account.dao.CompanyDao;
import com.uas.app.city.account.dao.LocalUserDao;
import com.uas.app.city.account.model.City;
import com.uas.app.city.account.model.Company;
import com.uas.app.city.account.model.LocalUser;
import com.uas.app.city.account.service.CityService;
import com.uas.app.city.account.service.CompanyService;
import com.uas.app.city.account.support.SystemSession;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import org.apache.commons.lang3.Validate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

/**
 * An implementation class for {@code CityService}.
 *
 * @author huxz
 */
@Service
public class CityServiceImpl implements CityService {

    private final CityDao cityDao;

    private final CompanyDao companyDao;

    private final CompanyService companyService;

    private final LocalUserDao userDao;

    @Autowired
    public CityServiceImpl(CityDao cityDao, CompanyDao companyDao,
            CompanyService companyService, LocalUserDao userDao) {
        this.cityDao = cityDao;
        this.companyDao = companyDao;
        this.companyService = companyService;
        this.userDao = userDao;
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public City insertCityInfoWhenAdminAdds(City city) {
        Validate.notNull(city, "City info can't be null");

        // Check whether current city has registered
        City existCity = cityDao.findByName(city.getName());
        if (existCity != null) {
            throw new IllegalStateException(String.format("City %s has registered", city.getName()));
        }

        city.setRegDate(new Date());
        city = cityDao.save(city);

        Company company = companyDao.findByBusinessCode(city.getBusinessCode());
        if (company == null) {
            company = companyService.findUserSpaceByBusinessCode(city.getBusinessCode());
        }

        Set<City> cities = company.getCities();
        cities.add(city);
        company.setCities(cities);
        companyDao.save(company);

        return city;
    }

    @Override
    public Page<City> pageCityOnConditions(Pageable pageable) {
        Validate.notNull(pageable, "Pageable can't be null");

        return cityDao.findAll(pageable);
    }

    @Override
    public City updateCityInfoWhenAdminEdit(City city) {
        Validate.notNull(city, "City info can't be null");

        return cityDao.save(city);
    }

    @Override
    public City deleteCityInfoByAdmin(Long cityId) {
        Validate.notNull(cityId, "City id can't be null");

        City city = cityDao.findOne(cityId);
        if (city == null) {
            return null;
        }

        // 取消个人用户关联
        Set<LocalUser> userSet = city.getUserSet();
        if (!CollectionUtils.isEmpty(userSet)) {
            for (LocalUser user : userSet) {
                user.getCities().remove(city);
                userDao.save(user);
            }
        }

        // 取消企业用户关联
        Set<Company> companySet = city.getCompanies();
        if (!CollectionUtils.isEmpty(companySet)) {
            for (Company company : companySet) {
                company.getCities().remove(city);
                companyDao.save(company);
            }
        }

        // 删除产城信息
        cityDao.delete(city);

        return city;
    }

    @Override
    public List<City> findCityListOfOneUser() {
        LocalUser user = SystemSession.getUser();
        Validate.notNull(user, "User is not login");

        if (user.getEnterprise() != null) {
            return new ArrayList<>(user.getEnterprise().getCities());
        } else {
            return new ArrayList<>(user.getCities());
        }
    }
}
