package com.uas.app.city.account.service.impl;

import com.uas.account.entity.User;
import com.uas.app.city.account.dao.CityDao;
import com.uas.app.city.account.dao.CompanyDao;
import com.uas.app.city.account.dao.LocalUserDao;
import com.uas.app.city.account.model.City;
import com.uas.app.city.account.model.Company;
import com.uas.app.city.account.model.LocalUser;
import com.uas.app.city.account.model.UserView;
import com.uas.app.city.account.service.CompanyService;
import com.uas.app.city.account.service.LocalUserService;
import com.uas.app.city.account.util.FlexJsonUtils;
import com.uas.sso.SSOToken;
import com.uas.sso.entity.UserAccount;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.Validate;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.Set;

/**
 * An implementation class for {@code LocalUserService}.
 *
 * @author huxz
 */
@Service
public class LocalUserServiceImpl implements LocalUserService {

    private static final Logger logger = Logger.getLogger(LocalUserServiceImpl.class);

    private final LocalUserDao userDao;

    private final CompanyDao companyDao;

    private final CompanyService companyService;

    private final CityDao cityDao;

    @Autowired
    public LocalUserServiceImpl(LocalUserDao userDao,
            CompanyDao companyDao, CompanyService companyService,
            CityDao cityDao) {
        this.userDao = userDao;
        this.companyDao = companyDao;
        this.companyService = companyService;
        this.cityDao = cityDao;
    }

    @Override
    @Transactional(rollbackFor = RuntimeException.class)
    public LocalUser findUserInfoByToken(SSOToken token, String cityId) {
        Validate.notNull(token, "SSO token can't be null after authentication");
        Validate.notBlank(cityId, "Id of current city must be non empty");

        UserAccount userAccount = FlexJsonUtils.fromJson(token.getData(), UserAccount.class);
        UserView tokenUser = new UserView(userAccount);
        if (logger.isDebugEnabled()) {
            logger.debug("Token user: " + tokenUser);
        }

        // 获取用户信息，用于会话更新
        LocalUser user = null;
        if (StringUtils.isNotEmpty(tokenUser.getDialectUID())) {
            user = userDao.findByUserUu(NumberUtils.toLong(tokenUser.getDialectUID()));
        }

        // 如果用户信息没有同步到本系统，则手动同步用户信息
        if (user == null && StringUtils.isNotEmpty(tokenUser.getDialectUID())) {
            // Synchronize token user info to this system when not find user info.
            user = new LocalUser();
            user.setUser(tokenUser);
            user.setUserUu(NumberUtils.toLong(tokenUser.getDialectUID()));

            if (StringUtils.isNotEmpty(tokenUser.getSpaceUID())) {
                Company company = saveCompany(tokenUser);

                Set<Company> enterprises = user.getEnterprises();
                enterprises.add(company);
                user.setEnterprises(enterprises);
            }

            userDao.save(user);

            if (logger.isDebugEnabled()) {
                logger.debug(String.format("User %s first login this app", tokenUser.getName()));
            }
        }

        // 如何当前登录用户为企业用户，获取企业信息
        if (user != null && StringUtils.isNotEmpty(tokenUser.getSpaceUID())) {
            Set<Company> companySet = user.getEnterprises();
            if (!CollectionUtils.isEmpty(companySet)) {
                for (Company company : user.getEnterprises()) {
                    if (tokenUser.getSpaceUID().equals(company.getBusinessCode())) {
                        user.setEnterprise(company);
                        break;
                    }
                }
            }

            // 如果企业用户当前企业信息为空，则绑定企业信息
            if (user.getEnterprise() == null) {
                Company company = saveCompany(tokenUser);

                Set<Company> enterprises = user.getEnterprises();
                enterprises.add(company);
                user.setEnterprises(enterprises);
                userDao.save(user);

                user.setEnterprise(company);
            }

            // 获取企业入驻产城信息
            Set<City> citySet = user.getEnterprise().getCities();
            for (City city : citySet) {
                if (city.getId().equals(NumberUtils.toLong(cityId))) {
                    user.setCurrentCity(city);
                    break;
                }
            }

            // 如果当前入驻产城信息为空，则绑定产城
            if (user.getCurrentCity() == null) {
                City city = cityDao.findOne(NumberUtils.toLong(cityId));
                Validate.notNull(city, "当前产城信息没有注册，请确认");

                citySet.add(city);
                user.getEnterprise().setCities(citySet);
                companyDao.save(user.getEnterprise());

                user.setCurrentCity(city);
            }
        } else if (user != null && !StringUtils.isNotEmpty(tokenUser.getSpaceUID())){
            // 获取个人用户入驻产城信息
            Set<City> citySet = user.getCities();
            for (City city : citySet) {
                if (city.getId().equals(NumberUtils.toLong(cityId))) {
                    user.setCurrentCity(city);
                    break;
                }
            }

            // 如果当前入驻产城信息为空，则绑定产城
            if (user.getCurrentCity() == null) {
                City city = cityDao.findOne(NumberUtils.toLong(cityId));
                Validate.notNull(city, "当前产城信息没有注册，请确认");

                citySet.add(city);
                user.setCities(citySet);
                userDao.save(user);

                user.setCurrentCity(city);
            }
        }

        if (user == null) {
            logger.error(String.format("invalid user %s, please set uid or dialectUID", tokenUser.getName()));
        }

        return user;
    }

    @Override
    public LocalUser save(LocalUser localUser) {
        LocalUser user = userDao.save(localUser);
        return user;
    }

    @Override
    public LocalUser findByUserUu(Long userUU) {
        LocalUser user = userDao.findByUserUu(userUU);
        return user;
    }

    private Company saveCompany(User tokenUser) {
        Company company = companyDao.findByBusinessCode(tokenUser.getSpaceUID());
        if (company == null) {
            company = companyService.findUserSpaceByBusinessCode(tokenUser.getSpaceUID());
            companyDao.save(company);
        }
        return company;
    }
}
