package com.uas.app.city.account.util;

import com.mashape.unirest.http.Headers;
import com.mashape.unirest.http.HttpResponse;
import com.mashape.unirest.http.JsonNode;
import com.mashape.unirest.http.Unirest;
import com.mashape.unirest.http.exceptions.UnirestException;
import com.uas.app.city.account.third.model.AccessToken;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.servlet.http.HttpServletRequest;
import org.apache.commons.lang3.Validate;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.log4j.Logger;

/**
 * Proxy request from browser or web client.
 *
 * @author huxz
 */
public class ProxyUtils {

    private static final Logger logger = Logger.getLogger(ProxyUtils.class);

    private static final String EMPTY_JSON_OBJECT = "{}";

    private static final String EMPTY_JSON_ARRAY = "[]";

    private static final String HTTPS_STARTS = "https";

    /**
     * Proxy request and return response body in json form.
     *
     * @param request       http request from browser
     * @param forwardUrl    proxy url
     * @param accessToken   access token
     * @return  data of response body
     */
    public static String proxyRequest(HttpServletRequest request, String forwardUrl,
            AccessToken accessToken)
            throws UnirestException {
        Validate.notNull(request, "Proxy request can't be null");
        Validate.notBlank(forwardUrl, "Forward url can't be empty");

        String path = forwardUrl + request.getRequestURI() + "?" + request.getQueryString()
                + "&access_token=" + accessToken.getId();
        if (logger.isDebugEnabled()) {
            logger.debug("Path: " + path);
        }

        return dispatchRequest(request, path);
    }

    private static String dispatchRequest(HttpServletRequest request, String path)
            throws UnirestException {
        final String requestMethod = request.getMethod();

        String result;
        switch (requestMethod) {
            case "GET":
                result = proxyGet(request, path);
                break;
            case "POST":
                result = proxyPost(request, path);
                break;
            default:
                result = EMPTY_JSON_OBJECT;
        }
        return result;
    }

    /**
     * Proxy request and return response body in json form.
     *
     * @param request       http request from browser
     * @param forwardUrl    proxy url
     * @return  data of response body
     */
    public static String proxyCommonRequest(HttpServletRequest request, String forwardUrl)
            throws UnirestException {
        Validate.notNull(request, "Proxy request can't be null");
        Validate.notBlank(forwardUrl, "Forward url can't be empty");

        String path = forwardUrl + request.getRequestURI() + "?" + request.getQueryString();
        if (logger.isDebugEnabled()) {
            logger.debug("Path: " + path);
        }

        return dispatchRequest(request, path);
    }

    private static String proxyGet(HttpServletRequest request, String path)
            throws UnirestException {

        HttpResponse<JsonNode> httpResponse = Unirest.get(path).header("Cookie", "").asJson();
        printResponseHeader(httpResponse.getHeaders());

        if (logger.isDebugEnabled()) {
            logger.debug("Result: " + httpResponse.getBody());
        }
        return handleJsonResponse(httpResponse);
    }

    private static String proxyPost(HttpServletRequest request, String path)
            throws UnirestException {

        Map<String, Object> parameters = getParameters(request);

        HttpResponse<JsonNode> httpResponse = Unirest.post(path).header("Cookie", "").queryString(parameters).asJson();
        printResponseHeader(httpResponse.getHeaders());

        if (logger.isDebugEnabled()) {
            logger.debug("Result: " + httpResponse.getBody());
        }
        return handleJsonResponse(httpResponse);
    }

    private static void printResponseHeader(Headers headers) {
        if (!headers.isEmpty()) {
            System.out.println("======================== Header ==========================");
            for (Entry<String, List<String>> next : headers.entrySet()) {
                System.out.println(next.getKey() + ": " + next.getValue());
            }
            System.out.println("======================== END ==========================");
        }
    }

    @Deprecated
    @SuppressWarnings("unused")
    private static CloseableHttpClient createHttpClientByUrl(String forwardUrl) {
        CloseableHttpClient httpClient;
        if (forwardUrl.startsWith(HTTPS_STARTS)) {
            SSLConnectionSocketFactory factory = getSslSocketFactory();
            httpClient = HttpClients.custom().setSSLSocketFactory(factory).build();
            Unirest.setHttpClient(httpClient);
        } else {
            httpClient = HttpClients.createDefault();
        }
        return httpClient;
    }

    private static SSLConnectionSocketFactory getSslSocketFactory() {
        SSLConnectionSocketFactory factory = null;
        try {
            SSLContext ctx = SSLContext.getInstance("TLS");
            X509TrustManager manager = new X509TrustManager() {
                @Override
                public void checkClientTrusted(X509Certificate[] x509Certificates, String s)
                        throws CertificateException {
                }

                @Override
                public void checkServerTrusted(X509Certificate[] x509Certificates, String s)
                        throws CertificateException {
                }

                @Override
                public X509Certificate[] getAcceptedIssuers() {
                    return new X509Certificate[0];
                }
            };
            ctx.init(null, new TrustManager[]{manager}, null);
            factory = new SSLConnectionSocketFactory(ctx,
                    SSLConnectionSocketFactory.getDefaultHostnameVerifier());
        } catch (NoSuchAlgorithmException | KeyManagementException e) {
            e.printStackTrace();
        }
        return factory;
    }

    private static String handleJsonResponse(HttpResponse<JsonNode> httpResponse) {
        JsonNode jsonNode = httpResponse.getBody();

        String result = EMPTY_JSON_OBJECT;
        if (jsonNode != null && jsonNode.isArray()) {
            result = jsonNode.getArray() != null ? jsonNode.getArray().toString() :
                    EMPTY_JSON_ARRAY;
        } else if (jsonNode != null && !jsonNode.isArray()) {
            result = jsonNode.getObject() != null ? jsonNode.getObject().toString() :
                    EMPTY_JSON_OBJECT;
        }
        return result;
    }

    private static Map<String, Object> getParameters(HttpServletRequest request) {
        Map<String, Object> queryString = new HashMap<>(16);
        Enumeration<String> parameterNames = request.getParameterNames();
        while (parameterNames.hasMoreElements()) {
            String parameterName = parameterNames.nextElement();
            if (!"cityId".equals(parameterName)) {
                queryString.put(parameterName, request.getParameter(parameterName));
            }
        }

        if (logger.isDebugEnabled()) {
            logger.debug("Parameters: " + queryString);
        }
        return queryString;
    }
}
